
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Carroll, Slacalek and Sommer: International Evidence on Sticky
% Consumption Growth
%
% This MATLAB program estimates the "true" consumption growth of section 3.2 with the Kalman
% smoother and produces table 4 and figures 1 and 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
kalmanPath='C:\Jiri\Research\intStickyC\paper\cssIntlStickyC20080215\programs\matlab';
addpath(kalmanPath); addpath([kalmanPath '\toolbox']);
cd(kalmanPath);
warning off;

delete([mfilename '.out']); diary([mfilename '.out']);
disp('==================================================================================');
disp(['Running  ' mfilename '.m                      ' datestr(now)]); format compact; tic;
disp('==================================================================================');

dataFile=[kalmanPath '\dataNoutput\data_kalman_ndss.txt'];
data=load(dataFile);
cStore=repmat(nan,188,13); cTrueStore=cStore; % Initialize matrices for C and C* (for graphs)

% Estimate "true" consumption growth
% Canada
%=========================================================================
c=data(6:188,2)-mean(data(6:188,2)); 
myoptions = optimset('fminsearch');
myoptions = optimset(myoptions, 'TolX', 1.0e-006 ,'TolFun', 1.0e-006, 'MaxFunEvals', 1000000, 'MaxIter', 1000000,'Display','off');
InitialValues=[0.1 log(0.0001/4) log(0.0001/400) 0]; % nondurables and services
[parameters, fval, exitflag, output, grad, hessian] = fminunc('Kalman_Likel', InitialValues, myoptions, c', 0);
% disp('LogLik Gamma VarianceObservation VarianceState Theta');
Kalman_Likel(parameters, c', 1);
disp('Canada')
disp('parameters, their standard errors, and t-statistics')
disp([parameters' sqrt(diag(inv(hessian))) parameters'./sqrt(diag(inv(hessian)))]);
%Write results to a table
line_number=1;
tableValues(line_number,1:4)=[parameters(1) parameters(4) parameters(2) parameters(3)];
load 'dataNoutput\results_kalman.mat' saveRatio xsmooth;
tableValues(line_number,5)=saveRatio;
tStats_temp=parameters'./sqrt(diag(inv(hessian)));
tStats(line_number,1:4)=[tStats_temp(1) tStats_temp(4) tStats_temp(2) tStats_temp(3)];
cStore(6:188,line_number)=c; cTrueStore(6:188,line_number)=xsmooth(1,:)';

% Germany
%=========================================================================
c=data(42:188,3)-mean(data(42:188,3)); 
myoptions = optimset('fminsearch');
myoptions = optimset(myoptions, 'TolX', 1.0e-006 ,'TolFun', 1.0e-006, 'MaxFunEvals', 1000000, 'MaxIter', 1000000,'Display','off');
InitialValues=[0.5 log(0.0001/4) log(0.0001/4) 0]; % nondurables and services
[parameters, fval, exitflag, output, grad, hessian] = fminunc('Kalman_Likel', InitialValues, myoptions, c', 0);
% disp('LogLik Gamma VarianceObservation VarianceState Theta');
Kalman_Likel(parameters, c', 1);
disp('Germany')
disp('parameters, their standard errors, and t-statistics')
disp([parameters' sqrt(diag(inv(hessian))) parameters'./sqrt(diag(inv(hessian)))]);
%Write results to a table
line_number=3;
tableValues(line_number,1:4)=[parameters(1) parameters(4) parameters(2) parameters(3)];
load 'dataNoutput\results_kalman.mat' saveRatio xsmooth;
tableValues(line_number,5)=saveRatio;
tStats_temp=parameters'./sqrt(diag(inv(hessian)));
tStats(line_number,1:4)=[tStats_temp(1) tStats_temp(4) tStats_temp(2) tStats_temp(3)];
cStore(42:188,line_number)=c; cTrueStore(42:188,line_number)=xsmooth(1,:)';

% France
%=========================================================================
c=data(74:188,4)-mean(data(74:188,4)); 
myoptions = optimset('fminsearch');
myoptions = optimset(myoptions, 'TolX', 1.0e-006 ,'TolFun', 1.0e-006, 'MaxFunEvals', 1000000, 'MaxIter', 1000000,'Display','off');
InitialValues=[0.5 log(0.00003/4) log(0.00003/400) 0]; % nondurables and services
[parameters, fval, exitflag, output, grad, hessian] = fminunc('Kalman_Likel', InitialValues, myoptions, c', 0);
% disp('LogLik Gamma VarianceObservation VarianceState Theta');
Kalman_Likel(parameters, c', 1);
disp('France')
disp('parameters, their standard errors, and t-statistics')
disp([parameters' sqrt(diag(inv(hessian))) parameters'./sqrt(diag(inv(hessian)))]);
%Write results to a table
line_number=2;
tableValues(line_number,1:4)=[parameters(1) parameters(4) parameters(2) parameters(3)];
load 'dataNoutput\results_kalman.mat' saveRatio xsmooth;
tableValues(line_number,5)=saveRatio;
tStats_temp=parameters'./sqrt(diag(inv(hessian)));
tStats(line_number,1:4)=[tStats_temp(1) tStats_temp(4) tStats_temp(2) tStats_temp(3)];
cStore(74:188,line_number)=c; cTrueStore(74:188,line_number)=xsmooth(1,:)';

% UK
%=========================================================================
c=data(18:187,5)-mean(data(18:187,5)); 
myoptions = optimset('fminsearch');
myoptions = optimset(myoptions, 'TolX', 1.0e-006 ,'TolFun', 1.0e-006, 'MaxFunEvals', 1000000, 'MaxIter', 1000000,'Display','off');
InitialValues=[0.5 log(0.0001/100) log(0.0001/100) 0]; % nondurables and services
[parameters, fval, exitflag, output, grad, hessian] = fminunc('Kalman_Likel', InitialValues, myoptions, c', 0);
% disp('LogLik Gamma VarianceObservation VarianceState Theta');
Kalman_Likel(parameters, c', 1);
disp('UK')
disp('parameters, their standard errors, and t-statistics')
disp([parameters' sqrt(diag(inv(hessian))) parameters'./sqrt(diag(inv(hessian)))]);
%Write results to a table
line_number=5;
tableValues(line_number,1:4)=[parameters(1) parameters(4) parameters(2) parameters(3)];
load 'dataNoutput\results_kalman.mat' saveRatio xsmooth;
tableValues(line_number,5)=saveRatio;
tStats_temp=parameters'./sqrt(diag(inv(hessian)));
tStats(line_number,1:4)=[tStats_temp(1) tStats_temp(4) tStats_temp(2) tStats_temp(3)];
cStore(18:187,line_number)=c; cTrueStore(18:187,line_number)=xsmooth(1,:)';

% Italy
%=========================================================================
c=data(86:187,6)-mean(data(86:187,6)); 
myoptions = optimset('fminsearch');
myoptions = optimset(myoptions, 'TolX', 1.0e-006 ,'TolFun', 1.0e-006, 'MaxFunEvals', 1000000, 'MaxIter', 1000000,'Display','off');
InitialValues=[0.5 log(0.0001/4) log(0.0001/400) 0]; % nondurables and services
[parameters, fval, exitflag, output, grad, hessian] = fminunc('Kalman_Likel', InitialValues, myoptions, c', 0);
% disp('LogLik Gamma VarianceObservation VarianceState Theta');
Kalman_Likel(parameters, c', 1);
disp('Italy')
disp('parameters, their standard errors, and t-statistics')
disp([parameters' sqrt(diag(inv(hessian))) parameters'./sqrt(diag(inv(hessian)))]);
%Write results to a table
line_number=4;
tableValues(line_number,1:4)=[parameters(1) parameters(4) parameters(2) parameters(3)];
load 'dataNoutput\results_kalman.mat' saveRatio xsmooth;
tableValues(line_number,5)=saveRatio;
tStats_temp=parameters'./sqrt(diag(inv(hessian)));
tStats(line_number,1:4)=[tStats_temp(1) tStats_temp(4) tStats_temp(2) tStats_temp(3)];
cStore(86:187,line_number)=c; cTrueStore(86:187,line_number)=xsmooth(1,:)';

% USA
%=========================================================================
c=data(2:188,8)-mean(data(2:188,8)); 
myoptions = optimset('fminsearch');
myoptions = optimset(myoptions, 'TolX', 1.0e-006 ,'TolFun', 1.0e-006, 'MaxFunEvals', 1000000, 'MaxIter', 1000000,'Display','off');
InitialValues=[0.5 log(0.0001/4) log(0.0001/4) 0]; % nondurables and services
[parameters, fval, exitflag, output, grad, hessian] = fminunc('Kalman_Likel', InitialValues, myoptions, c', 0);
% disp('LogLik Gamma VarianceObservation VarianceState Theta');
Kalman_Likel(parameters, c', 1);
disp('USA')
disp('parameters, their standard errors, and t-statistics')
disp([parameters' sqrt(diag(inv(hessian))) parameters'./sqrt(diag(inv(hessian)))]);
%Write results to a table
line_number=6;
tableValues(line_number,1:4)=[parameters(1) parameters(4) parameters(2) parameters(3)];
load 'dataNoutput\results_kalman.mat' saveRatio xsmooth;
tableValues(line_number,5)=saveRatio;
tStats_temp=parameters'./sqrt(diag(inv(hessian)));
tStats(line_number,1:4)=[tStats_temp(1) tStats_temp(4) tStats_temp(2) tStats_temp(3)];
cStore(2:188,line_number)=c; cTrueStore(2:188,line_number)=xsmooth(1,:)';

% Australia
%=========================================================================
c=data(23:176,1)-mean(data(23:176,1)); 
myoptions = optimset('fminsearch');
myoptions = optimset(myoptions, 'TolX', 1.0e-006 ,'TolFun', 1.0e-006, 'MaxFunEvals', 1000000, 'MaxIter', 1000000,'Display','off');
InitialValues=[0.5 log(0.0001/4) log(0.0001/400) 0]; % total consumption
[parameters, fval, exitflag, output, grad, hessian] = fminunc('Kalman_Likel', InitialValues, myoptions, c', 0);
% disp('LogLik Gamma VarianceObservation VarianceState Theta');
Kalman_Likel(parameters, c', 1);
disp('Australia')
disp('parameters, their standard errors, and t-statistics')
%disp([parameters' sqrt(diag(inv(hessian)))]);
disp([parameters' sqrt(diag(inv(hessian))) parameters'./sqrt(diag(inv(hessian)))]);
%Write results to a table
line_number=7;
tableValues(line_number,1:4)=[parameters(1) parameters(4) parameters(2) parameters(3)];
load 'dataNoutput\results_kalman.mat' saveRatio xsmooth;
tableValues(line_number,5)=saveRatio;
tStats_temp=parameters'./sqrt(diag(inv(hessian)));
tStats(line_number,1:4)=[tStats_temp(1) tStats_temp(4) tStats_temp(2) tStats_temp(3)];
cStore(23:176,line_number)=c; cTrueStore(23:176,line_number)=xsmooth(1,:)';

% Belgium
%=========================================================================
c=data(6:174,9)-mean(data(6:174,9)); 
myoptions = optimset('fminsearch');
myoptions = optimset(myoptions, 'TolX', 1.0e-006 ,'TolFun', 1.0e-006, 'MaxFunEvals', 1000000, 'MaxIter', 1000000,'Display','off');
InitialValues=[0.7 log(0.0001/4) log(0.0001/4) 0]; % total consumption
[parameters, fval, exitflag, output, grad, hessian] = fminunc('Kalman_Likel', InitialValues, myoptions, c', 0);
% disp('LogLik Gamma VarianceObservation VarianceState Theta');
Kalman_Likel(parameters, c', 1);
disp('Belgium')
disp('parameters, their standard errors, and t-statistics')
disp([parameters' sqrt(diag(inv(hessian))) parameters'./sqrt(diag(inv(hessian)))]);
%Write results to a table
line_number=8;
tableValues(line_number,1:4)=[parameters(1) parameters(4) parameters(2) parameters(3)];
load 'dataNoutput\results_kalman.mat' saveRatio xsmooth;
tableValues(line_number,5)=saveRatio;
tStats_temp=parameters'./sqrt(diag(inv(hessian)));
tStats(line_number,1:4)=[tStats_temp(1) tStats_temp(4) tStats_temp(2) tStats_temp(3)];
cStore(6:174,line_number)=c; cTrueStore(6:174,line_number)=xsmooth(1,:)';

% Denmark
%=========================================================================
c=data(6:174,10)-mean(data(6:174,10)); 
myoptions = optimset('fminsearch');
myoptions = optimset(myoptions, 'TolX', 1.0e-006 ,'TolFun', 1.0e-006, 'MaxFunEvals', 1000000, 'MaxIter', 1000000,'Display','off');
InitialValues=[0.5 log(0.0001/4) log(0.0001/4) 0]; % total consumption
[parameters, fval, exitflag, output, grad, hessian] = fminunc('Kalman_Likel', InitialValues, myoptions, c', 0);
% disp('LogLik Gamma VarianceObservation VarianceState Theta');
Kalman_Likel(parameters, c', 1);
disp('Denmark')
disp('parameters, their standard errors, and t-statistics')
disp([parameters' sqrt(diag(inv(hessian))) parameters'./sqrt(diag(inv(hessian)))]);
%Write results to a table
line_number=9;
tableValues(line_number,1:4)=[parameters(1) parameters(4) parameters(2) parameters(3)];
load 'dataNoutput\results_kalman.mat' saveRatio xsmooth;
tableValues(line_number,5)=saveRatio;
tStats_temp=parameters'./sqrt(diag(inv(hessian)));
tStats(line_number,1:4)=[tStats_temp(1) tStats_temp(4) tStats_temp(2) tStats_temp(3)];
cStore(6:174,line_number)=c; cTrueStore(6:174,line_number)=xsmooth(1,:)';

% Finland
%=========================================================================
c=data(6:174,11)-mean(data(6:174,11)); 
myoptions = optimset('fminsearch');
myoptions = optimset(myoptions, 'TolX', 1.0e-006 ,'TolFun', 1.0e-006, 'MaxFunEvals', 1000000, 'MaxIter', 1000000,'Display','off');
InitialValues=[0.5 log(0.0001/4) log(0.0001/4) 0]; % total consumption
[parameters, fval, exitflag, output, grad, hessian] = fminunc('Kalman_Likel', InitialValues, myoptions, c', 0);
% disp('LogLik Gamma VarianceObservation VarianceState Theta');
Kalman_Likel(parameters, c', 1);
disp('Finland')
disp('parameters, their standard errors, and t-statistics')
disp([parameters' sqrt(diag(inv(hessian))) parameters'./sqrt(diag(inv(hessian)))]);
%Write results to a table
line_number=10;
tableValues(line_number,1:4)=[parameters(1) parameters(4) parameters(2) parameters(3)];
load 'dataNoutput\results_kalman.mat' saveRatio xsmooth;
tableValues(line_number,5)=saveRatio;
tStats_temp=parameters'./sqrt(diag(inv(hessian)));
tStats(line_number,1:4)=[tStats_temp(1) tStats_temp(4) tStats_temp(2) tStats_temp(3)];
cStore(6:174,line_number)=c; cTrueStore(6:174,line_number)=xsmooth(1,:)';

% Netherlands
%=========================================================================
c=data(6:178,12)-mean(data(6:178,12)); 
myoptions = optimset('fminsearch');
myoptions = optimset(myoptions, 'TolX', 1.0e-006 ,'TolFun', 1.0e-006, 'MaxFunEvals', 1000000, 'MaxIter', 1000000,'Display','off');
InitialValues=[0.5 log(0.0001/4) log(0.0001/4) 0]; % total consumption
[parameters, fval, exitflag, output, grad, hessian] = fminunc('Kalman_Likel', InitialValues, myoptions, c', 0);
% disp('LogLik Gamma VarianceObservation VarianceState Theta');
Kalman_Likel(parameters, c', 1);
disp('Netherlands')
disp('parameters, their standard errors, and t-statistics')
disp([parameters' sqrt(diag(inv(hessian))) parameters'./sqrt(diag(inv(hessian)))]);
%Write results to a table
line_number=11;
tableValues(line_number,1:4)=[parameters(1) parameters(4) parameters(2) parameters(3)];
load 'dataNoutput\results_kalman.mat' saveRatio xsmooth;
tableValues(line_number,5)=saveRatio;
tStats_temp=parameters'./sqrt(diag(inv(hessian)));
tStats(line_number,1:4)=[tStats_temp(1) tStats_temp(4) tStats_temp(2) tStats_temp(3)];
cStore(6:178,line_number)=c; cTrueStore(6:178,line_number)=xsmooth(1,:)';

% Sweden
%=========================================================================
c=data(6:174,13)-mean(data(6:174,13)); 
myoptions = optimset('fminsearch');
myoptions = optimset(myoptions, 'TolX', 1.0e-006 ,'TolFun', 1.0e-006, 'MaxFunEvals', 1000000, 'MaxIter', 1000000,'Display','off');
InitialValues=[0.5 log(0.0001/4) log(0.0001/4) 0]; % total consumption
[parameters, fval, exitflag, output, grad, hessian] = fminunc('Kalman_Likel', InitialValues, myoptions, c', 0);
% disp('LogLik Gamma VarianceObservation VarianceState Theta');
Kalman_Likel(parameters, c', 1);
disp('Sweden')
disp('parameters, their standard errors, and t-statistics')
disp([parameters' sqrt(diag(inv(hessian))) parameters'./sqrt(diag(inv(hessian)))]);
%Write results to a table
line_number=13;
tableValues(line_number,1:4)=[parameters(1) parameters(4) parameters(2) parameters(3)];
load 'dataNoutput\results_kalman.mat' saveRatio xsmooth;
tableValues(line_number,5)=saveRatio;
tStats_temp=parameters'./sqrt(diag(inv(hessian)));
tStats(line_number,1:4)=[tStats_temp(1) tStats_temp(4) tStats_temp(2) tStats_temp(3)];
cStore(6:174,line_number)=c; cTrueStore(6:174,line_number)=xsmooth(1,:)';

% Spain
%=========================================================================
c=data(11:174,14)-mean(data(11:174,14)); % 11
myoptions = optimset('fminsearch');
myoptions = optimset(myoptions, 'TolX', 1.0e-006 ,'TolFun', 1.0e-006, 'MaxFunEvals', 1000000, 'MaxIter', 1000000,'Display','off');
InitialValues=[0.5 log(0.0001/4) log(0.0001/4) 0]; % total consumption
[parameters, fval, exitflag, output, grad, hessian] = fminunc('Kalman_Likel', InitialValues, myoptions, c', 0);
% disp('LogLik Gamma VarianceObservation VarianceState Theta');
Kalman_Likel(parameters, c', 1);
disp('Spain')
disp('parameters, their standard errors, and t-statistics')
disp([parameters' sqrt(diag(inv(hessian))) parameters'./sqrt(diag(inv(hessian)))]);
%Write results to a table
line_number=12;
tableValues(line_number,1:4)=[parameters(1) parameters(4) parameters(2) parameters(3)];
load 'dataNoutput\results_kalman.mat' saveRatio xsmooth;
tableValues(line_number,5)=saveRatio;
tStats_temp=parameters'./sqrt(diag(inv(hessian)));
tStats(line_number,1:4)=[tStats_temp(1) tStats_temp(4) tStats_temp(2) tStats_temp(3)];
cStore(11:174,line_number)=c; cTrueStore(11:174,line_number)=xsmooth(1,:)';

save 'dataNoutput\results_kalman.mat';
write1stStage;  % write Table 4

% Draw figures 1 and 2
%=========================================================================
time=(1960:.25:(2007-.01))';
countryNames=strvcat('Canada','France','Germany','Italy','United Kingdom','United States','Australia','Belgium','Denmark','Finland', ...
    'Netherlands','Spain','Sweden');

fG7=figure;
for i=1:6;
    subplot(4,2,i);
    h=plot(time,[cTrueStore(:,i) cStore(:,i)]);
    set(h, 'color','k'); set(h(1),'linewidth',2); set(h(2),'linewidth',0.5);
    ttt=title(countryNames(i,:)); set(ttt,'fontsize',9); axis tight; axis 'auto y'; axis([-inf inf -0.04 0.04]); 
    set(gca,'fontsize',6);
    if i==2; lll=legend('True','Measured',3); set(lll,'fontsize',6); end;
end;

print(fG7,'-depsc','.\dataNoutput\fG7.eps');

fOther=figure;
for i=7:13;
    subplot(4,2,(i-6));
    h=plot(time,[cTrueStore(:,i) cStore(:,i)]);
    set(h, 'color','k'); set(h(1),'linewidth',2); set(h(2),'linewidth',0.5);
    ttt=title(countryNames(i,:)); set(ttt,'fontsize',9); axis tight; axis 'auto y'; axis([-inf inf -0.04 0.04]); 
    set(gca,'fontsize',6);
    if i==8; lll=legend('True','Measured',3); set(lll,'fontsize',6); end;
end;

print(fOther,'-depsc',strcat(kalmanPath,'.\dataNoutput\fOther.eps'));

%==========================================================================
t=toc; mints=floor(toc/60); secs=toc-60*mints;
fprintf('Time: %4.0f minutes, %4.2f seconds\n',mints,secs); diary off;

